package com.muyu.aliyun.bailian;

import com.aliyun.sdk.service.bailian20231229.AsyncClient;
import com.aliyun.sdk.service.bailian20231229.models.ApplyFileUploadLeaseRequest;
import com.aliyun.sdk.service.bailian20231229.models.ApplyFileUploadLeaseResponse;
import com.aliyun.sdk.service.bailian20231229.models.ApplyFileUploadLeaseResponseBody;
import com.aliyun.tea.*;
import com.muyu.aliyun.bailian.data.FileUploadLeaseMode;
import com.muyu.aliyun.bailian.data.fixed.UploadLeaseParamsMode;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;

@Component
@Log4j2
public class ApplyLease {

    @Autowired
    private AsyncClient asyncClient;

    public UploadLeaseParamsMode applyFileUploadLease(FileUploadLeaseMode fileUploadLeaseModel) {
        ApplyFileUploadLeaseRequest applyFileUploadLeaseRequest = ApplyFileUploadLeaseRequest.builder()
                .fileName(fileUploadLeaseModel.getFileName())
                .md5(fileUploadLeaseModel.getMd5())
                .sizeInBytes(fileUploadLeaseModel.getSizeInBytes())
                .categoryId(fileUploadLeaseModel.getCategoryId())
                .workspaceId(fileUploadLeaseModel.getWorkspaceId())
                .build();
        String exceptionName = "";
        try {
            // 复制代码运行请自行打印 API 的返回值
            CompletableFuture<ApplyFileUploadLeaseResponse> applyFileUploadLeaseResponseCompletableFuture = asyncClient.applyFileUploadLease(applyFileUploadLeaseRequest);
            ApplyFileUploadLeaseResponse applyFileUploadLeaseResponse = applyFileUploadLeaseResponseCompletableFuture.get();
            if (applyFileUploadLeaseResponse.getStatusCode() != 200) {
                throw new RuntimeException(applyFileUploadLeaseResponse.toString());
            }
            ApplyFileUploadLeaseResponseBody applyFileUploadLeaseResponseBody = applyFileUploadLeaseResponse.getBody();
            ApplyFileUploadLeaseResponseBody.Param param = applyFileUploadLeaseResponseBody.getData().getParam();


            log.info("param:「{}」", param);
            return UploadLeaseParamsMode.builder()
                    .Url(param.getUrl())
                    .Method(param.getMethod())
                    .build();


        }  catch (TeaException error) {
            switch (error.getCode()) {
                case "SDK.ConnectTimeout":
                    exceptionName = "网络连接超时";
                    break;
                case "SDK.ReadTimeout":
                    exceptionName = "网络读取超时";
                    break;
                case "SDK.ServerUnreachable":
                    exceptionName = "服务器不可达";
                    break;
                case "InvalidAccessKeyId.NotFound":
                    exceptionName = "无效的访问密钥 ID";
                    break;
                case "SignatureDoesNotMatch":
                    exceptionName = "签名不匹配";
                    break;
                case "MissingParameter":
                    exceptionName = "缺少必要参数";
                    break;
                case "InvalidParameterValue":
                    exceptionName = "参数值无效";
                    break;
                case "Forbidden.NoPermission":
                    exceptionName = "没有权限";
                    break;
                default:
                    exceptionName = "未知阿里云 API 异常";
            }
            log.error("异常名称：「{}」，错误码: {}, 错误信息: {}, 错误数据: {}", exceptionName, error.getCode(), error.getMessage(), error.getData());
        }catch (Exception exception) {
            log.error("异常名称：「未知通用异常」，错误信息: {}", exception.getMessage(), exception);
            throw new RuntimeException(exception);
        }
        throw new RuntimeException(String.format("API调用异常：「%S」",exceptionName));
    }

}
